/******************************************************************************
* remove_superfluous_vertices.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/

#ifndef REMOVE_SUPERFLUOUS_VERTICES_H_
#define REMOVE_SUPERFLUOUS_VERTICES_H_

#include "../dataStructures/graph.h"
#include "../geometry/tools.h"

#include <vector>

namespace schematization {
//removes vertices the exact same coordinates and are adjacent

template<typename schemGraphType>
class remove_superfluous_vertices {
public:
	// The main routine to bypass intersections
	static size_t run(schemGraphType& input_graph) {
		///***   +
        /*  {
		graph_traits<schemGraphType>::edge_iterator e;
				BGL_FORALL_EDGES(e, input_graph, schemGraphType){
        	    cout << "found edge: " << e << endl;
	        	}
	        	}
  	        	cout << "num_edges: " << num_edges(input_graph) << endl;
	        	cout << "num_vertices: " << num_vertices(input_graph) << endl;
        */
        
        
        bool doublette_found = true;
        while (doublette_found){
        doublette_found = false;
        typename graph_traits<schemGraphType>::vertex_iterator vertex_a, last_v;
        typename graph_traits<schemGraphType>::vertex_iterator vertex_b, last_v_2;
        
       
		for (tie(vertex_a, last_v) = vertices(input_graph);vertex_a != last_v; ++vertex_a) {
    		for (tie(vertex_b, last_v) = vertices(input_graph);vertex_b != last_v; ++vertex_b) {

    		    if (*vertex_b == *vertex_a)
    		        continue;
		        
		    if ( (input_graph[*vertex_a].coord.x == input_graph[*vertex_b].coord.x) &&
		        (input_graph[*vertex_a].coord.y == input_graph[*vertex_b].coord.y) )
		        {
                    /*
		            cout << endl << "doublette gefunden" << endl;
		            cout << *vertex_a << ", " << *vertex_b << endl;
		            cout << input_graph[*vertex_a].coord.x << ", " << input_graph[*vertex_a].coord.y << " | " << input_graph[*vertex_b].coord.x << ", " << input_graph[*vertex_b].coord.y << endl;
		            */
		            
		            //determine if vertices are adjacent
		            bool vertices_adjacent = false;
		            typename graph_traits<schemGraphType>::edge_iterator context_e;
            		BGL_FORALL_OUTEDGES_T(*vertex_a, context_e, input_graph, schemGraphType){
            		    if (target(context_e, input_graph) == *vertex_b)
                		    vertices_adjacent = true;
            		}
            		BGL_FORALL_OUTEDGES_T(*vertex_b, context_e, input_graph, schemGraphType){
            		    if (target(context_e, input_graph) == *vertex_a)
                		    vertices_adjacent = true;
            		}
            		
            		//cout << "vertices are adjacent: " << vertices_adjacent << endl;
            		if (!vertices_adjacent)
            		    continue;
		            
		            doublette_found = true;
		            
		            /*
		                Nodes A and B are too close (i.e., their coordinates match)
		                
                        1. remove edge between A and B
                        2. add all edges to A that are incident to B		            
                        3. remove B
                    */
		            
		            
		            //identify and remove edge between A and B
		        	vector <typename schemGraphType::edge_descriptor> remove_edge_descriptors;
		            typename graph_traits<schemGraphType>::edge_iterator e;
            		BGL_FORALL_EDGES_T(e, input_graph, schemGraphType){
			        	if (input_graph[e].context) continue;
			        	if (target(e, input_graph) == *vertex_a && source(e, input_graph) == *vertex_b){
			        	    //cout << "found edge: " << e << endl;
			        	    //remove_edge(e, input_graph);
			        	    remove_edge_descriptors.push_back(e);
			        	}
			        	else if (target(e, input_graph) == *vertex_b && source(e, input_graph) == *vertex_a){
			        	    remove_edge_descriptors.push_back(e);
			        	}
			        	
		        	}
		        	
		        	for (int l = 0; l < remove_edge_descriptors.size(); l++)
		        	    remove_edge(remove_edge_descriptors[l], input_graph);		        	
		        	
		        	remove_edge_descriptors.clear();
		        	
		        	   
		        	///step 2
		            typename graph_traits<schemGraphType>::edge_iterator e2;
		            typename schemGraphType::edge_descriptor edge_desc;

		        	
		        	BGL_FORALL_EDGES_T(e2, input_graph, schemGraphType) {

			            	if (target(e2, input_graph) == *vertex_a){
			            	    
			            	    if (source(e2, input_graph) == *vertex_b)
			            	        continue;
			            	        
		                	    bool edge_created;
                				tie(edge_desc, edge_created) = add_edge(source(e2, input_graph), *vertex_b, input_graph);  //
		                	    input_graph[edge_desc].name = input_graph[e2].name;
		                	    input_graph[edge_desc].context = input_graph[e2].context;
		                	    input_graph[edge_desc].category = input_graph[e2].category;			            	
		                	    remove_edge_descriptors.push_back(e2);  			      
 			            	}
			            	else if (source(e2, input_graph) == *vertex_a)
			            	{
			            	    if (target(e2, input_graph) == *vertex_b)
			            	        continue;
			            	        
		                	    bool edge_created;
                				tie(edge_desc, edge_created) = add_edge(*vertex_b, target(e2, input_graph), input_graph);  //
		                	    input_graph[edge_desc].name = input_graph[e2].name;
		                	    input_graph[edge_desc].context = input_graph[e2].context;
		                	    input_graph[edge_desc].category = input_graph[e2].category;
		                	    remove_edge_descriptors.push_back(e2);  			      
			            	}
		        	}
                
                        		        	
		        	for (int l = 0; l < remove_edge_descriptors.size(); l++)
		        	    remove_edge(remove_edge_descriptors[l], input_graph);
	        	    

                    //step 3: remove *vertex_a
                    remove_vertex(*vertex_a, input_graph);
		        	break;
		        	
		        }
		        
		        }
		        if (doublette_found)
		            break;
		    }
		    
		    return 0;
		}
		/*
        {
		graph_traits<schemGraphType>::edge_iterator e;
				BGL_FORALL_EDGES(e, input_graph, schemGraphType){
        	    cout << "found edge: " << e << endl;
	        	}
	        	}
	        	cout << "num_edges: " << num_edges(input_graph) << endl;
	        	cout << "num_vertices: " << num_vertices(input_graph) << endl;
	        	*/
		///***
	}



private:

};

}

#endif 
