/******************************************************************************
* point.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * point.h
 *
 *  Created on: 09.06.2010
 *      Author: pajor
 */

#ifndef POINT_H_
#define POINT_H_

#include "../types.h"

#include <iomanip>

namespace schematization {

struct point {
	double x;
	double y;
	point() : x(0.0), y(0.0) {}
	point(double x_, double y_) : x(x_), y(y_) {}

	// Test for equivalency
	inline bool equivalent_to(const point &other) const {
		return round(x) == round(other.x) && round(y) == round(other.y);
	}

	// Some operators
	point operator-(const point &other) {
		return point(this->x-other.x, this->y-other.y);
	}
	point operator+(const point &other) {
		return point(this->x+other.x, this->y+other.y);
	}

	bool operator==(const point &other) const {
		return (this->x == other.x && this->y == other.y);
	}

	friend ostream &operator<<(ostream &os, const point &p) {
		return os << setprecision(16) << p.x << " " << p.y;
	}

	friend istream &operator>>(istream &is, point &p) {
		is.setf(std::ios_base::skipws);
		return is >> p.x >> p.y;
	}
};

}

#endif /* POINT_H_ */
